/*
 * $Header: /home/cvspublic/jakarta-slide/proposals/taglib/src/common/org/apache/slide/taglib/tag/NodeTagSupport.java,v 1.3 2004/07/30 06:51:48 ozeigermann Exp $
 * $Revision: 1.3 $
 * $Date: 2004/07/30 06:51:48 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. the names "The Jakarta Project", "Slide", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.taglib.tag;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.slide.common.SlideException;
import org.apache.slide.taglib.bean.NamespaceBean;
import org.apache.slide.taglib.bean.NodeBean;

/**
 * Abstract foundation for tags that serve to select a particular 
 * <code>ObjectNode</code> in a Slide namespace (identified by the node's URI),
 * and expose a corresponding 
 * {@link org.apache.slide.taglib.bean.NodeBean NodeBean} as scripting 
 * variable.
 * 
 * @author <a href="mailto:cmlenz@apache.org">Christopher Lenz</a>
 * @version $Revision: 1.3 $
 */
public abstract class NodeTagSupport
    extends TagSupport {
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * The associated bean that will be put in the PageContext attributes.
     */
    protected NodeBean node;
    
    
    /**
     * Value of 'uri' attribute.
     */
    protected String uri;
    
    
    /**
     * Value of 'resolveLinks' attribute.
     */
    protected String resolveLinks;
    
    
    /**
     * Name of the PageContext attribute under which the bean should be stored.
     */
    protected String attrName;
    
    
    // ----------------------------------------------------------- Construction
    
    
    /**
     * Initialize inherited and local state.
     */
    public NodeTagSupport() {
        super();
        
        init();
    }
    
    
    // ------------------------------------------------------------ Tag Methods
    
    
    /**
     * Called by the JSP Engine when opening this tag.
     *
     * @throws JspException not thrown
     */
    public int doStartTag()
        throws JspException {
        super.doStartTag();
        
        if (node != null) {
            if (attrName != null) {
                pageContext.setAttribute(attrName, node);
            }
            
            return EVAL_BODY_INCLUDE;
        }
        
        return SKIP_BODY;
    }
    
    
    /**
     * Called by the JSP Engine when closing of this tag.
     *
     * @throws JspException not thrown
     */
    public int doEndTag()
        throws JspException {
        super.doEndTag();
        
        if (attrName != null) {
            pageContext.removeAttribute(attrName);
        }
        init(); // ensure the tag can be reused
        
        return EVAL_PAGE;
    }
    
    
    /**
     * Releases any resources we may have (or inherit).
     */
    public void release() {
        super.release();
        
        init();
    }
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Sets the 'resolveLinks' attribute.
     *
     * @param resolveLinks the attribute value
     */
    public void setResolveLinks(String resolveLinks) {
        
        this.resolveLinks = resolveLinks;
    }
    
    
    /**
     * Sets the 'uri' attribute.
     *
     * @param uri the attribute value
     */
    public void setUri(String uri) {
        
        this.uri = uri;
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Returns the bean representing the node selected by this tag.
     *
     * @return the NodeBean
     */
    public NodeBean getNode() {
        
        return node;
    }
    
    
    // ------------------------------------------------------ Protected Methods
    
    
    /**
     * Convenience method to support subclasses in retrieving the chosen node
     * from the namespace, and wrapping it in a 
     * {@link org.apache.slide.taglib.bean.NodeBean NodeBean}.
     * 
     * @param namespace the bean that represents the selected namespace
     * @param uri the URI of the node to retrieve
     * @param resolveLinks whether to resolve links when retrieving the node
     */
    protected NodeBean getNode(NamespaceBean namespace, String uri,
                               boolean resolveLinks) {
        
        NodeBean bean = null;
        if ((namespace != null) && (uri != null)) {
            try {
                bean = namespace.getNode(uri, resolveLinks);
            } catch (SlideException e) {
                // ignore for now
            }
        }
        
        return bean;
    }
    
    
    // -------------------------------------------------------- Private Methods
    
    
    /**
     * Reset internal state.
     */
    private void init() {
        
        node = null;
        uri = null;
        resolveLinks = null;
        attrName = null;
    }
    
    
}


